/*
 * Copyright (C) 2010 SHARP Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jp.co.sharp.android.samples.pedometer;

import jp.co.sharp.android.hardware.Pedometer;
import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.InflateException;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
/**
 * 歩数計サンプルアプリのメインActivityである。<br />
 * 歩数計APIのメソッドを利用し、搭載されている歩数計から計測した値を取得して表示する。<br />
 * メニューからその機種でサポートしているデータ種別の一覧表を開く。
 */
public class SamplePedometerActivity extends Activity implements OnClickListener {

    // Pedometerクラスのオブジェクト
    private Pedometer mPedometer;
    // StartボタンでIntent通知を開始しているか判定するフラグ
    private boolean mPedometerStartFlg;
    // 画面再描画時のBundleに保持させる値のkey値
    private static final String START = "start";
    private static final String STEP = "step";
    private static final String KM = "km";
    private static final String KCAL = "kcal";

    /**
     * ブロードキャストレシーバー。
     * startPedometerReceiver()内で設定された周期で
     * 歩数計から値を取得する。
     */
    private BroadcastReceiver mReceiver= new BroadcastReceiver(){
        @Override
        public void onReceive(Context context, Intent intent) {

            String action = intent.getAction();

            Log.e("mReceiver", "Intent " + System.currentTimeMillis());
            if(Pedometer.ACTION_MEASURE.equals(action)) {
                // intの値をインテントからデータ取得
                int steps = intent.getIntExtra(Pedometer.STEPS, 0);

                // floatの値をインテントからデータ取得
                float calorie = intent.getFloatExtra(Pedometer.CALORIE, 0);
                float distance = intent.getFloatExtra(Pedometer.DISTANCE, 0);

                // 画面に表示
                setText(steps, calorie, distance);
            }
        }
    };

    /**
     * 画面起動時に呼ばれる。
     * onCreate()をオーバーライドしている。<br />
     * 歩数計非対応の場合は非対応画面を表示させる。
     * @param bundle Bundle
     */
    @Override
    public void onCreate(Bundle bundle) {
        super.onCreate(bundle);

        try{
        // Pedometerクラスを取得
        mPedometer = Pedometer.createInstance(this);    /* ★Addon用API使用箇所★ */

        } catch (NoClassDefFoundError e) {
            // 非対応(APIなし/歩数計機能なし)機種の場合
            setContentView(R.layout.main_not_support);
        	return;
        }

        // StartボタンによるIntent通知開始フラグの初期化
        mPedometerStartFlg = false;

        if (null != mPedometer) {
            // 対応機種の場合
            setContentView(R.layout.main);

            // ボタンにクリックリスナーを登録する
            Button start = (Button)findViewById(R.id.StartStopButton);
            start.setOnClickListener(this);
            Button update = (Button)findViewById(R.id.UpdateButton);
            update.setOnClickListener(this);

        } else {
            // 非対応(APIあり/歩数計機能なし)機種の場合
            setContentView(R.layout.main_not_support);
        }

    }

    /**
     * 画面が前面に出た場合の処理する。
     * onResume()をオーバーライドしている。<br />
     * Stopボタン状態（Intent受信状態で中断した場合）だとIntent受信要求をだす
     */
    @Override
    protected void onResume() {
        super.onResume();

        if (null != mPedometer) {
            // 対応機種画面の場合
            if(mPedometerStartFlg){
                // Startボタンから起動された状態（フラグがTrue）でonPause()の停止を受けた場合は再開
                // フラグを戻してからIntent通知開始（StartボタンでIntent通知を開始するときフラグはFalseのため）
                mPedometerStartFlg = !mPedometerStartFlg;
                startPedometerReceiver();
            }
        }

    }

    /**
     * 画面が隠れた場合の処理です。
     * Startボタンが押されたままであればIntentの受信をやめる。
     */
    @Override
    protected void onPause() {
        super.onPause();
        // Activityが完全に非表示になったらIntentの受信をやめる。
        if (null != mPedometer) {
            // 対応機種の場合
            stopPedometerReceiver();    /* ★Addon用API使用箇所★ */
        }
    }

    /**
     * 画面が再描画される場合に備えてActivityの状態を保持しておく処理
     * onSaveInstanceState()をオーバーライドしている。<br />
     * 1回でも画面に測定値を表示していた場合は画面に表示していたSteps、Km、kcalの最後の値を保持しておく。<br />
     * Intent通知開始状態の場合はフラグの状態を保持しておく。
     */
    @Override
    protected void onSaveInstanceState(Bundle bundle) {
        super.onSaveInstanceState(bundle);
        if (null != mPedometer) {
            // 対応機種の場合
            // Intent通知の開始状態フラグを保存しておく
            bundle.putBoolean(START, mPedometerStartFlg);
            TextView step = (TextView)findViewById(R.id.hosuu);

            if (!mPedometerStartFlg && 0 < step.getText().length()) {
                // Intent通知は停止状態でも画面に数値が出ている場合はその値を保持
                TextView km = (TextView)findViewById(R.id.kyori);
                TextView kcal = (TextView)findViewById(R.id.cal);
                bundle.putInt(STEP, Integer.parseInt(step.getText().toString()));
                bundle.putFloat(KM, Float.parseFloat(km.getText().toString()));
                bundle.putFloat(KCAL, Float.parseFloat(kcal.getText().toString()));
            } else {
                // 何もしてないとき
                bundle.putInt(STEP, -1);
            }

        }
    }

    /**
     * 画面が再描画される場合に保持しておいたActivityの状態を再開する処理です。<br />
     * onRestoreInstanceState()をオーバーライドしている。
     */
    @Override
    protected void onRestoreInstanceState(Bundle bundle) {
        super.onRestoreInstanceState(bundle);
        if (null != mPedometer) {
            // 対応機種の場合
            if(null != bundle) {
                if(bundle.getBoolean(START)) {
                    // 保持しておいたフラグがtrue（Intent通知開始状態）であったらIntent通知を再開させる
                    mPedometerStartFlg = bundle.getBoolean(START);
                } else {
                    // Intent通知開始状態でなくとも1回以上起動していれば画面に描画していた数値を再表示
                    if (-1 != bundle.getInt(STEP)) {
                        TextView step = (TextView)findViewById(R.id.hosuu);
                        TextView km = (TextView)findViewById(R.id.kyori);
                        TextView kcal = (TextView)findViewById(R.id.cal);

                        step.setText(String.valueOf(bundle.getInt(STEP)));
                        km.setText(String.valueOf(bundle.getFloat(KM)));
                        kcal.setText(String.valueOf(bundle.getFloat(KCAL)));
                    }
                }
            }
        }
    }


    /**
     * ボタンがクリックされた時の処理
     *
     * STARTボタン   : Intent受信処理の開始
     * STOPボタン    : Intent受信処理の終了
     * UPDATEボタン  : 歩数、距離、消費カロリー値を取得する
     * @param view クリックされたボタンのID
     */
    @Override
    public void onClick(View view) {
        Button start = (Button)findViewById(R.id.StartStopButton);
        Button update = (Button)findViewById(R.id.UpdateButton);

        if(view == start) {
            // STARTまたはSTOPボタンがクリックされた時
            if(!mPedometerStartFlg) {
                startPedometerReceiver();
            } else {
                stopPedometerReceiver();
                // ボタン操作によりIntent通知の終了を行うとフラグの値が変わる
                mPedometerStartFlg = !mPedometerStartFlg;
            }
        } else if(view == update) {
            // UPDATEボタンがクリックされた時
            getPedometerParam();
        }
    }

    /**
     * STARTとSTOPのボタン表記を切り替える処理
     */
    private void changeButton() {
        Button start = (Button)findViewById(R.id.StartStopButton);
        if(!mPedometerStartFlg) {
            start.setText(R.string.stop);
            // ボタン操作によりIntent通知の開始/終了を行うとフラグの値が変わる
            mPedometerStartFlg = !mPedometerStartFlg;
        } else {
            start.setText(R.string.start);
        }
    }

    /**
     * Intentの受信処理を開始する。
     * 歩数計アプリが起動していない場合は、その旨をToastで表示する。
     */
    private void startPedometerReceiver() {
        // Pedometerクラスを取得
        mPedometer = Pedometer.createInstance(this);    /* ★Addon用API使用箇所★ */
        String[] device = { Pedometer.MEASURE_STATUS };
        Bundle data = mPedometer.getParameters(device);    /* ★Addon用API使用箇所★ */

        if(data.getInt(Pedometer.MEASURE_STATUS) == Pedometer.DEVICE_STOPED) {
            // 歩数計が起動していない場合は、起動していないことを通知する
            Toast.makeText(this, getString(R.string.please_pedometer_on), Toast.LENGTH_LONG).show();
        } else {
            IntentFilter filter = new IntentFilter();

            // サービスからのインテントを500msごとに受けるように設定
            filter.addAction(Pedometer.ACTION_MEASURE);

            //レシーバーの登録
            registerReceiver(mReceiver,filter);

            //インテント通知要求
            mPedometer.startBroadcastIntent();    /* ★Addon用API使用箇所★ */
            changeButton();
        }
    }

    /**
     * Intentの受信処理を終了する。
     * すでに終了済みもしくは、受信処理を開始していない場合は何もしない。
     */
    private void stopPedometerReceiver() {
        if(mPedometerStartFlg){
            // Intent通知終了要求
            mPedometer.stopBroadcastIntent();    /* ★Addon用API使用箇所★ */

            // レシーバーの登録解除
            unregisterReceiver(mReceiver);
            changeButton();
        }
    }

    /**
     * 歩数計から、歩数、消費カロリー、距離のデータを取得して
     * 各TextViewに表示させるメソッドを呼び出す。
     */
    private void getPedometerParam() {
        //取得したいデータの識別を配列にする。
        String[] keys = {
                Pedometer.STEPS,
                Pedometer.CALORIE,
                Pedometer.DISTANCE,
        };

        //指定したデータを取得
        Bundle data = mPedometer.getParameters(keys);    /* ★Addon用API使用箇所★ */

        // intの値を取得
        int steps = data.getInt(Pedometer.STEPS);

        // floatの値を取得
        float calorie  = data.getFloat(Pedometer.CALORIE, 0);
        float distance = data.getFloat(Pedometer.DISTANCE);

        //画面に表示
        setText(steps, calorie, distance);
    }

    /**
     * 各TextViewに歩数計から取得した値を設定する。
     * @param steps 歩数
     * @param calorie 消費カロリー
     * @param distance 距離
     */
    private void setText(int steps, float calorie, float distance) {
        TextView step = (TextView)findViewById(R.id.hosuu);
        TextView km = (TextView)findViewById(R.id.kyori);
        TextView kcal = (TextView)findViewById(R.id.cal);

        step.setText(String.valueOf(steps));
        kcal.setText(String.valueOf(calorie));
        km.setText(String.valueOf(distance));
    }

    /**
     * オプションメニュー表示時の処理
     * onPrepareOptionsMenuをオーバーライドします。
     *
     * @param menu Menu
     * @return trueを返却します。
     */
    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        if(null != mPedometer) {
            menu.findItem(R.id.optionMenu1_SupportDataList);
        }
        return true;
    }

    /**
     * オプションメニューの生成
     * onCreateOptionsMenuをオーバーライドします。
     *
     * @param oMenu Menu
     * @return 表示/非表示
     */
    @Override
    public boolean onCreateOptionsMenu(Menu oMenu) {
        if(null != mPedometer) {
            try {
                MenuInflater inflater = getMenuInflater();
                inflater.inflate(R.menu.optionmenu, oMenu);
            } catch (InflateException e) {
                e.printStackTrace();
            }
        }
        return true;
    }

    /**
     * メニューアイテムが選択された時の処理
     * onMenuItemSelectedをオーバーライドします。
     *
     * @param featureId アイテムのID
     * @param item メニューアイテム
     * @return 処理の結果
     */
    @Override
    public boolean onMenuItemSelected(int featureId, MenuItem item) {
        if(null != mPedometer) {
            int id = item.getItemId();
            if (id == R.id.optionMenu1_SupportDataList) {
                // サポート情報一覧へ遷移させる
                Intent intent = new Intent(this, SamplePedometerSupportDataList.class);
                startActivity(intent);
                return true;
            }
            if (id == R.id.optionMenu2_PedometerSetting) {
                // 歩数計設定画面へ遷移させる
                String className =
                    "jp.co.sharp.android.settings.pedometersetting.PedometerSettingActivity";
                String packageName = "com.android.settings";

                Intent intent = new Intent(Intent.ACTION_MAIN);
                intent.setClassName(packageName, className);

                /* 歩数計設定Activityを呼び出す
                 * エミュレーターではインスタンスの取得ができても
                 * 歩数計設定Activityが居ないため、Activityが存在しない場合は
                 * 歩数計設定が起動できないことをToastとして出す
                 */
                try {
                    startActivity(intent);
                } catch (ActivityNotFoundException e) {
                    Toast.makeText(this, getString(R.string.canot_pedmeter_setting), Toast.LENGTH_LONG).show();
                }
                return true;

            }
        }
        return false;
    }

}
